@@ -3,6 +3,8 @@
 
 """Base test case class for coverage.py testing."""
 
+from __future__ import annotations
+
 import contextlib
 import datetime
 import difflib
@@ -15,12 +17,17 @@ import re
 import shlex
 import sys
 
-import pytest
+from types import ModuleType
+from typing import (
+    Any, Collection, Iterable, Iterator, Mapping, Sequence,
+)
 
 import coverage
-from coverage import env
+from coverage import Coverage
 from coverage.cmdline import CoverageScript
+from coverage.data import CoverageData
 from coverage.misc import import_local_file
+from coverage.types import TArc, TLineNo
 
 from tests.helpers import arcs_to_arcz_repr, arcz_to_arcs, assert_count_equal
 from tests.helpers import nice_file, run_command
@@ -56,15 +63,20 @@ class CoverageTest(
     # Let stderr go to stderr, pytest will capture it for us.
     show_stderr = True
 
-    def setUp(self):
+    def setUp(self) -> None:
         super().setUp()
 
         # Attributes for getting info about what happened.
-        self.last_command_status = None
-        self.last_command_output = None
-        self.last_module_name = None
+        self.last_command_status: int | None = None
+        self.last_command_output: str | None = None
+        self.last_module_name: str | None = None
 
-    def start_import_stop(self, cov, modname, modfile=None):
+    def start_import_stop(
+        self,
+        cov: Coverage,
+        modname: str,
+        modfile: str | None = None,
+    ) -> ModuleType:
         """Start coverage, import a file, then stop coverage.
 
         `cov` is started and stopped, with an `import_local_file` of
@@ -74,6 +86,20 @@ class CoverageTest(
         The imported module is returned.
 
         """
+        # Here's something I don't understand. I tried changing the code to use
+        # the handy context manager, like this:
+        #
+        #   with cov.collect():
+        #       # Import the Python file, executing it.
+        #       return import_local_file(modname, modfile)
+        #
+        # That seemed to work, until 7.4.0 when it made metacov fail after
+        # running all the tests.  The deep recursion tests in test_oddball.py
+        # seemed to cause something to be off so that a "Trace function
+        # changed" error would happen as pytest was cleaning up, failing the
+        # metacov runs.  Putting back the old code below fixes it, but I don't
+        # understand the difference.
+
         cov.start()
         try:                                    # pragma: nested
             # Import the Python file, executing it.
@@ -83,22 +109,28 @@ class CoverageTest(
             cov.stop()
         return mod
 
-    def get_report(self, cov, squeeze=True, **kwargs):
+    def get_report(self, cov: Coverage, squeeze: bool = True, **kwargs: Any) -> str:
         """Get the report from `cov`, and canonicalize it."""
         repout = io.StringIO()
         kwargs.setdefault("show_missing", False)
         cov.report(file=repout, **kwargs)
         report = repout.getvalue().replace('\\', '/')
+        print(report)   # When tests fail, it's helpful to see the output
         if squeeze:
             report = re.sub(r" +", " ", report)
         return report
 
-    def get_module_name(self):
+    def get_module_name(self) -> str:
         """Return a random module name to use for this test run."""
         self.last_module_name = 'coverage_test_' + str(random.random())[2:]
         return self.last_module_name
 
-    def _check_arcs(self, a1, a2, arc_type):
+    def _check_arcs(
+        self,
+        a1: Iterable[TArc] | None,
+        a2: Iterable[TArc] | None,
+        arc_type: str,
+    ) -> str:
         """Check that the arc lists `a1` and `a2` are equal.
 
         If they are equal, return empty string. If they are unequal, return
@@ -116,11 +148,20 @@ class CoverageTest(
             return ""
 
     def check_coverage(
-        self, text, lines=None, missing="", report="",
-        excludes=None, partials="",
-        arcz=None, arcz_missing=None, arcz_unpredicted=None,
-        arcs=None, arcs_missing=None, arcs_unpredicted=None,
-    ):
+        self,
+        text: str,
+        lines: Sequence[TLineNo] | Sequence[list[TLineNo]] | None = None,
+        missing: str = "",
+        report: str = "",
+        excludes: Iterable[str] | None = None,
+        partials: Iterable[str] = (),
+        arcz: str | None = None,
+        arcz_missing: str | None = None,
+        arcz_unpredicted: str | None = None,
+        arcs: Iterable[TArc] | None = None,
+        arcs_missing: Iterable[TArc] | None = None,
+        arcs_unpredicted: Iterable[TArc] | None = None,
+    ) -> Coverage:
         """Check the coverage measurement of `text`.
 
         The source `text` is run and measured.  `lines` are the line numbers
@@ -170,11 +211,11 @@ class CoverageTest(
         # Get the analysis results, and check that they are right.
         analysis = cov._analyze(mod)
         statements = sorted(analysis.statements)
-        if lines is not None:
+        if lines:
             if isinstance(lines[0], int):
                 # lines is just a list of numbers, it must match the statements
                 # found in the code.
-                assert statements == lines, f"{statements!r} != {lines!r}"
+                assert statements == lines, f"lines: {statements!r} != {lines!r}"
             else:
                 # lines is a list of possible line number lists, one of them
                 # must match.
@@ -185,25 +226,18 @@ class CoverageTest(
                     assert False, f"None of the lines choices matched {statements!r}"
 
             missing_formatted = analysis.missing_formatted()
-            if isinstance(missing, str):
-                msg = f"{missing_formatted!r} != {missing!r}"
-                assert missing_formatted == missing, msg
-            else:
-                for missing_list in missing:
-                    if missing_formatted == missing_list:
-                        break
-                else:
-                    assert False, f"None of the missing choices matched {missing_formatted!r}"
+            msg = f"missing: {missing_formatted!r} != {missing!r}"
+            assert missing_formatted == missing, msg
 
         if arcs is not None:
             # print("Possible arcs:")
             # print(" expected:", arcs)
-            # print(" actual:", analysis.arc_possibilities())
+            # print(" actual:", analysis.arc_possibilities)
             # print("Executed:")
-            # print(" actual:", sorted(set(analysis.arcs_executed())))
+            # print(" actual:", sorted(set(analysis.arcs_executed)))
             # TODO: this would be nicer with pytest-check, once we can run that.
             msg = (
-                self._check_arcs(arcs, analysis.arc_possibilities(), "Possible") +
+                self._check_arcs(arcs, analysis.arc_possibilities, "Possible") +
                 self._check_arcs(arcs_missing, analysis.arcs_missing(), "Missing") +
                 self._check_arcs(arcs_unpredicted, analysis.arcs_unpredicted(), "Unpredicted")
             )
@@ -218,18 +252,34 @@ class CoverageTest(
 
         return cov
 
-    def make_data_file(self, basename=None, suffix=None, lines=None, file_tracers=None):
+    def make_data_file(
+        self,
+        basename: str | None = None,
+        *,
+        suffix: str | None = None,
+        lines: Mapping[str, Collection[TLineNo]] | None = None,
+        arcs: Mapping[str, Collection[TArc]] | None = None,
+        file_tracers: Mapping[str, str] | None = None,
+    ) -> CoverageData:
         """Write some data into a coverage data file."""
         data = coverage.CoverageData(basename=basename, suffix=suffix)
+        assert lines is None or arcs is None
         if lines:
             data.add_lines(lines)
+        if arcs:
+            data.add_arcs(arcs)
         if file_tracers:
             data.add_file_tracers(file_tracers)
         data.write()
         return data
 
     @contextlib.contextmanager
-    def assert_warnings(self, cov, warnings, not_warnings=()):
+    def assert_warnings(
+        self,
+        cov: Coverage,
+        warnings: Iterable[str],
+        not_warnings: Iterable[str] = (),
+    ) -> Iterator[None]:
         """A context manager to check that particular warnings happened in `cov`.
 
         `cov` is a Coverage instance.  `warnings` is a list of regexes.  Every
@@ -247,7 +297,11 @@ class CoverageTest(
         """
         __tracebackhide__ = True
         saved_warnings = []
-        def capture_warning(msg, slug=None, once=False):        # pylint: disable=unused-argument
+        def capture_warning(
+            msg: str,
+            slug: str | None = None,
+            once: bool = False,                 # pylint: disable=unused-argument
+        ) -> None:
             """A fake implementation of Coverage._warn, to capture warnings."""
             # NOTE: we don't implement `once`.
             if slug:
@@ -255,7 +309,7 @@ class CoverageTest(
             saved_warnings.append(msg)
 
         original_warn = cov._warn
-        cov._warn = capture_warning
+        cov._warn = capture_warning             # type: ignore[method-assign]
 
         try:
             yield
@@ -280,36 +334,41 @@ class CoverageTest(
                 if saved_warnings:
                     assert False, f"Unexpected warnings: {saved_warnings!r}"
         finally:
-            cov._warn = original_warn
+            cov._warn = original_warn           # type: ignore[method-assign]
 
-    def assert_same_files(self, flist1, flist2):
+    def assert_same_files(self, flist1: Iterable[str], flist2: Iterable[str]) -> None:
         """Assert that `flist1` and `flist2` are the same set of file names."""
         flist1_nice = [nice_file(f) for f in flist1]
         flist2_nice = [nice_file(f) for f in flist2]
         assert_count_equal(flist1_nice, flist2_nice)
 
-    def assert_exists(self, fname):
+    def assert_exists(self, fname: str) -> None:
         """Assert that `fname` is a file that exists."""
         assert os.path.exists(fname), f"File {fname!r} should exist"
 
-    def assert_doesnt_exist(self, fname):
+    def assert_doesnt_exist(self, fname: str) -> None:
         """Assert that `fname` is a file that doesn't exist."""
         assert not os.path.exists(fname), f"File {fname!r} shouldn't exist"
 
-    def assert_file_count(self, pattern, count):
+    def assert_file_count(self, pattern: str, count: int) -> None:
         """Assert that there are `count` files matching `pattern`."""
         files = sorted(glob.glob(pattern))
         msg = "There should be {} files matching {!r}, but there are these: {}"
         msg = msg.format(count, pattern, files)
         assert len(files) == count, msg
 
-    def assert_recent_datetime(self, dt, seconds=10, msg=None):
+    def assert_recent_datetime(
+        self,
+        dt: datetime.datetime,
+        seconds: int = 10,
+        msg: str | None = None,
+    ) -> None:
         """Assert that `dt` marks a time at most `seconds` seconds ago."""
         age = datetime.datetime.now() - dt
         assert age.total_seconds() >= 0, msg
         assert age.total_seconds() <= seconds, msg
 
-    def command_line(self, args, ret=OK):
+    def command_line(self, args: str, ret: int = OK) -> None:
         """Run `args` through the command line.
 
         Use this when you want to run the full coverage machinery, but in the
@@ -330,7 +389,7 @@ class CoverageTest(
     # https://salsa.debian.org/debian/pkg-python-coverage/-/blob/master/debian/patches/02.rename-public-programs.patch
     coverage_command = "coverage"
 
-    def run_command(self, cmd):
+    def run_command(self, cmd: str) -> str:
         """Run the command-line `cmd` in a sub-process.
 
         `cmd` is the command line to invoke in a sub-process. Returns the
@@ -347,7 +406,7 @@ class CoverageTest(
         _, output = self.run_command_status(cmd)
         return output
 
-    def run_command_status(self, cmd):
+    def run_command_status(self, cmd: str) -> tuple[int, str]:
         """Run the command-line `cmd` in a sub-process, and print its output.
 
         Use this when you need to test the process behavior of coverage.
@@ -382,72 +441,57 @@ class CoverageTest(
             command_words = [os.path.basename(sys.executable)]
 
         elif command_name == "coverage":
-            if env.JYTHON:                  # pragma: only jython
-                # Jython can't do reporting, so let's skip the test now.
-                if command_args and command_args[0] in ('report', 'html', 'xml', 'annotate'):
-                    pytest.skip("Can't run reporting commands in Jython")
-                # Jython can't run "coverage" as a command because the shebang
-                # refers to another shebang'd Python script. So run them as
-                # modules.
-                command_words = "jython -m coverage".split()
-            else:
-                # The invocation requests the coverage.py program.  Substitute the
-                # actual coverage.py main command name.
-                command_words = [self.coverage_command]
+            # The invocation requests the coverage.py program.  Substitute the
+            # actual coverage.py main command name.
+            command_words = [self.coverage_command]
 
         else:
             command_words = [command_name]
 
         cmd = " ".join([shlex.quote(w) for w in command_words] + command_args)
 
-        # Add our test modules directory to PYTHONPATH.  I'm sure there's too
-        # much path munging here, but...
-        pythonpath_name = "PYTHONPATH"
-        if env.JYTHON:
-            pythonpath_name = "JYTHONPATH"          # pragma: only jython
-
-        testmods = nice_file(self.working_root(), "tests/modules")
-        zipfile = nice_file(self.working_root(), "tests/zipmods.zip")
-        pypath = os.getenv(pythonpath_name, '')
-        if pypath:
-            pypath += os.pathsep
-        pypath += testmods + os.pathsep + zipfile
-        self.set_environ(pythonpath_name, pypath)
-
         self.last_command_status, self.last_command_output = run_command(cmd)
         print(self.last_command_output)
         return self.last_command_status, self.last_command_output
 
-    def working_root(self):
+    def add_test_modules_to_pythonpath(self) -> None:
+        """Add our test modules directory to PYTHONPATH."""
+        # Check that there isn't already a PYTHONPATH.
+        assert os.getenv("PYTHONPATH") is None
+        testmods = nice_file(self.working_root(), "tests/modules")
+        zipfile = nice_file(self.working_root(), "tests/zipmods.zip")
+        self.set_environ("PYTHONPATH", testmods + os.pathsep + zipfile)
+
+    def working_root(self) -> str:
         """Where is the root of the coverage.py working tree?"""
         return os.path.dirname(nice_file(__file__, ".."))
 
-    def report_from_command(self, cmd):
+    def report_from_command(self, cmd: str) -> str:
         """Return the report from the `cmd`, with some convenience added."""
         report = self.run_command(cmd).replace('\\', '/')
         assert "error" not in report.lower()
         return report
 
-    def report_lines(self, report):
+    def report_lines(self, report: str) -> list[str]:
         """Return the lines of the report, as a list."""
         lines = report.split('\n')
         assert lines[-1] == ""
         return lines[:-1]
 
-    def line_count(self, report):
+    def line_count(self, report: str) -> int:
         """How many lines are in `report`?"""
         return len(self.report_lines(report))
 
-    def squeezed_lines(self, report):
+    def squeezed_lines(self, report: str) -> list[str]:
         """Return a list of the lines in report, with the spaces squeezed."""
         lines = self.report_lines(report)
         return [re.sub(r"\s+", " ", l.strip()) for l in lines]
 
-    def last_line_squeezed(self, report):
+    def last_line_squeezed(self, report: str) -> str:
         """Return the last line of `report` with the spaces squeezed down."""
         return self.squeezed_lines(report)[-1]
 
-    def get_measured_filenames(self, coverage_data):
+    def get_measured_filenames(self, coverage_data: CoverageData) -> dict[str, str]:
         """Get paths to measured files.
 
         Returns a dict of {filename: absolute path to file}
@@ -456,20 +500,21 @@ class CoverageTest(
         return {os.path.basename(filename): filename
                 for filename in coverage_data.measured_files()}
 
-    def get_missing_arc_description(self, cov, start, end):
+    def get_missing_arc_description(self, cov: Coverage, start: TLineNo, end: TLineNo) -> str:
         """Get the missing-arc description for a line arc in a coverage run."""
         # ugh, unexposed methods??
+        assert self.last_module_name is not None
         filename = self.last_module_name + ".py"
         fr = cov._get_file_reporter(filename)
-        arcs_executed = cov._analyze(filename).arcs_executed()
+        arcs_executed = cov._analyze(filename).arcs_executed
         return fr.missing_arc_description(start, end, arcs_executed)
 
 
 class UsingModulesMixin:
     """A mixin for importing modules from tests/modules and tests/moremodules."""
 
-    def setUp(self):
-        super().setUp()
+    def setUp(self) -> None:
+        super().setUp()             # type: ignore[misc]
 
         # Parent class saves and restores sys.path, we can just modify it.
         sys.path.append(nice_file(TESTS_DIR, "modules"))
@@ -477,7 +522,7 @@ class UsingModulesMixin:
         sys.path.append(nice_file(TESTS_DIR, "zipmods.zip"))
 
 
-def command_line(args):
+def command_line(args: str) -> int:
     """Run `args` through the CoverageScript command line.
 
     Returns the return code from CoverageScript.command_line.
