@@ -13,7 +13,7 @@ import os.path
 import re
 import xml.etree.ElementTree
 
-from typing import Iterable, List, Optional, Tuple
+from typing import Iterable
 
 from tests.coveragetest import TESTS_DIR
 from tests.helpers import os_sep
@@ -27,9 +27,9 @@ def gold_path(path: str) -> str:
 def compare(
     expected_dir: str,
     actual_dir: str,
-    file_pattern: Optional[str] = None,
+    file_pattern: str | None = None,
     actual_extra: bool = False,
-    scrubs: Optional[List[Tuple[str, str]]] = None,
+    scrubs: list[tuple[str, str]] | None = None,
 ) -> None:
     """Compare files matching `file_pattern` in `expected_dir` and `actual_dir`.
 
@@ -91,6 +91,8 @@ def compare(
             print(f":::: diff '{expected_file}' and '{actual_file}'")
             print("\n".join(difflib.Differ().compare(expected_lines, actual_lines)))
             print(f":::: end diff '{expected_file}' and '{actual_file}'")
+            print(f"==== expected output in '{os.path.abspath(expected_dir)}'")
+            print(f"==== actual output in '{os.path.abspath(actual_dir)}'")
             save_mismatch(f)
 
     if not actual_extra:
@@ -99,9 +101,9 @@ def compare(
 
     assert not text_diff, "Files differ: " + "\n".join(text_diff)
 
-    assert not expected_only, f"Files in {expected_dir} only: {expected_only}"
+    assert not expected_only, f"Files in {os.path.abspath(expected_dir)} only: {expected_only}"
     if not actual_extra:
-        assert not actual_only, f"Files in {actual_dir} only: {actual_only}"
+        assert not actual_only, f"Files in {os.path.abspath(actual_dir)} only: {actual_only}"
 
 
 def contains(filename: str, *strlist: str) -> None:
@@ -175,7 +177,7 @@ def canonicalize_xml(xtext: str) -> str:
     return xml.etree.ElementTree.tostring(root).decode("utf-8")
 
 
-def _fnmatch_list(files: List[str], file_pattern: Optional[str]) -> List[str]:
+def _fnmatch_list(files: list[str], file_pattern: str | None) -> list[str]:
     """Filter the list of `files` to only those that match `file_pattern`.
     If `file_pattern` is None, then return the entire list of files.
     Returns a list of the filtered files.
@@ -185,7 +187,7 @@ def _fnmatch_list(files: List[str], file_pattern: Optional[str]) -> List[str]:
     return files
 
 
-def scrub(strdata: str, scrubs: Iterable[Tuple[str, str]]) -> str:
+def scrub(strdata: str, scrubs: Iterable[tuple[str, str]]) -> str:
     """Scrub uninteresting data from the payload in `strdata`.
     `scrubs` is a list of (find, replace) pairs of regexes that are used on
     `strdata`.  A string is returned.
