@@ -7,11 +7,15 @@ Test class mixins
 Some of these are transitional while working toward pure-pytest style.
 """
 
+from __future__ import annotations
+
 import importlib
 import os
 import os.path
 import sys
 
+from typing import Any, Callable, Iterable, Iterator, Tuple, cast
+
 import pytest
 
 from coverage.misc import SysModuleSaver
@@ -22,26 +26,30 @@ class PytestBase:
     """A base class to connect to pytest in a test class hierarchy."""
 
     @pytest.fixture(autouse=True)
-    def connect_to_pytest(self, request, monkeypatch):
+    def connect_to_pytest(
+        self,
+        request: pytest.FixtureRequest,
+        monkeypatch: pytest.MonkeyPatch,
+    ) -> None:
         """Captures pytest facilities for use by other test helpers."""
         # pylint: disable=attribute-defined-outside-init
         self._pytest_request = request
         self._monkeypatch = monkeypatch
         self.setUp()
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Per-test initialization. Override this as you wish."""
         pass
 
-    def addCleanup(self, fn, *args):
+    def addCleanup(self, fn: Callable[..., None], *args: Any) -> None:
         """Like unittest's addCleanup: code to call when the test is done."""
         self._pytest_request.addfinalizer(lambda: fn(*args))
 
-    def set_environ(self, name, value):
+    def set_environ(self, name: str, value: str) -> None:
         """Set an environment variable `name` to be `value`."""
         self._monkeypatch.setenv(name, value)
 
-    def del_environ(self, name):
+    def del_environ(self, name: str) -> None:
         """Delete an environment variable, unless we set it."""
         self._monkeypatch.delenv(name, raising=False)
 
@@ -55,10 +63,10 @@ class TempDirMixin:
     run_in_temp_dir = True
 
     @pytest.fixture(autouse=True)
-    def _temp_dir(self, tmpdir_factory):
+    def _temp_dir(self, tmp_path_factory: pytest.TempPathFactory) -> Iterator[None]:
         """Create a temp dir for the tests, if they want it."""
         if self.run_in_temp_dir:
-            tmpdir = tmpdir_factory.mktemp("t")
+            tmpdir = tmp_path_factory.mktemp("t")
             self.temp_dir = str(tmpdir)
             with change_dir(self.temp_dir):
                 # Modules should be importable from this temp directory.  We don't
@@ -70,7 +78,13 @@ class TempDirMixin:
         else:
             yield
 
-    def make_file(self, filename, text="", bytes=b"", newline=None):
+    def make_file(
+        self,
+        filename: str,
+        text: str = "",
+        bytes: bytes = b"",
+        newline: str | None = None,
+    ) -> str:
         """Make a file. See `tests.helpers.make_file`"""
         # pylint: disable=redefined-builtin     # bytes
         assert self.run_in_temp_dir, "Only use make_file when running in a temp dir"
@@ -81,7 +95,7 @@ class RestoreModulesMixin:
     """Auto-restore the imported modules at the end of each test."""
 
     @pytest.fixture(autouse=True)
-    def _module_saving(self):
+    def _module_saving(self) -> Iterable[None]:
         """Remove modules we imported during the test."""
         self._sys_module_saver = SysModuleSaver()
         try:
@@ -89,7 +103,7 @@ class RestoreModulesMixin:
         finally:
             self._sys_module_saver.restore()
 
-    def clean_local_file_imports(self):
+    def clean_local_file_imports(self) -> None:
         """Clean up the results of calls to `import_local_file`.
 
         Use this if you need to `import_local_file` the same file twice in
@@ -99,7 +113,7 @@ class RestoreModulesMixin:
         # So that we can re-import files, clean them out first.
         self._sys_module_saver.restore()
 
-        # Also have to clean out the .pyc files, since the timestamp
+        # Also have to clean out the .pyc files, since the time stamp
         # resolution is only one second, a changed file might not be
         # picked up.
         remove_tree("__pycache__")
@@ -118,18 +132,18 @@ class StdStreamCapturingMixin:
 
     """
     @pytest.fixture(autouse=True)
-    def _capcapsys(self, capsys):
+    def _capcapsys(self, capsys: pytest.CaptureFixture[str]) -> None:
         """Grab the fixture so our methods can use it."""
         self.capsys = capsys
 
-    def stdouterr(self):
+    def stdouterr(self) -> tuple[str, str]:
         """Returns (out, err), two strings for stdout and stderr."""
-        return self.capsys.readouterr()
+        return cast(Tuple[str, str], self.capsys.readouterr())
 
-    def stdout(self):
+    def stdout(self) -> str:
         """Returns a string, the captured stdout."""
         return self.capsys.readouterr().out
 
-    def stderr(self):
+    def stderr(self) -> str:
         """Returns a string, the captured stderr."""
         return self.capsys.readouterr().err
