@@ -3,14 +3,21 @@
 
 """Tests for concurrency libraries."""
 
+from __future__ import annotations
+
 import glob
+import multiprocessing
 import os
+import pathlib
 import random
 import re
 import sys
 import threading
 import time
 
+from types import ModuleType
+from typing import Iterable
+
 from flaky import flaky
 import pytest
 
@@ -21,16 +28,13 @@ from coverage.exceptions import ConfigError
 from coverage.files import abs_file
 from coverage.misc import import_local_file
 
+from tests import testenv
 from tests.coveragetest import CoverageTest
+from tests.helpers import flaky_method
 
 
 # These libraries aren't always available, we'll skip tests if they aren't.
 
-try:
-    import multiprocessing
-except ImportError:         # pragma: only jython
-    multiprocessing = None
-
 try:
     import eventlet
 except ImportError:
@@ -43,11 +47,11 @@ except ImportError:
 
 try:
     import greenlet
-except ImportError:         # pragma: only jython
+except ImportError:
     greenlet = None
 
 
-def measurable_line(l):
+def measurable_line(l: str) -> bool:
     """Is this a line of code coverage will measure?
 
     Not blank, not a comment, and not "else"
@@ -59,18 +63,15 @@ def measurable_line(l):
         return False
     if l.startswith('else:'):
         return False
-    if env.JYTHON and l.startswith(('try:', 'except:', 'except ', 'break', 'with ')):
-        # Jython doesn't measure these statements.
-        return False                    # pragma: only jython
     return True
 
 
-def line_count(s):
+def line_count(s: str) -> int:
     """How many measurable lines are in `s`?"""
     return len(list(filter(measurable_line, s.splitlines())))
 
 
-def print_simple_annotation(code, linenos):
+def print_simple_annotation(code: str, linenos: Iterable[int]) -> None:
     """Print the lines in `code` with X for each line number in `linenos`."""
     for lineno, line in enumerate(code.splitlines(), start=1):
         print(" {} {}".format("X" if lineno in linenos else " ", line))
@@ -81,7 +82,7 @@ class LineCountTest(CoverageTest):
 
     run_in_temp_dir = False
 
-    def test_line_count(self):
+    def test_line_count(self) -> None:
         CODE = """
             # Hey there!
             x = 1
@@ -175,7 +176,7 @@ SIMPLE = """
     """
 
 
-def cant_trace_msg(concurrency, the_module):
+def cant_trace_msg(concurrency: str, the_module: ModuleType | None) -> str | None:
     """What might coverage.py say about a concurrency setting and imported module?"""
     # In the concurrency choices, "multiprocessing" doesn't count, so remove it.
     if "multiprocessing" in concurrency:
@@ -189,7 +190,7 @@ def cant_trace_msg(concurrency, the_module):
         expected_out = (
             f"Couldn't trace with concurrency={concurrency}, the module isn't installed.\n"
         )
-    elif env.C_TRACER or concurrency == "thread" or concurrency == "":
+    elif testenv.C_TRACER or concurrency == "thread" or concurrency == "":
         expected_out = None
     else:
         expected_out = (
@@ -203,7 +204,13 @@ class ConcurrencyTest(CoverageTest):
 
     QLIMIT = 1000
 
-    def try_some_code(self, code, concurrency, the_module, expected_out=None):
+    def try_some_code(
+        self,
+        code: str,
+        concurrency: str,
+        the_module: ModuleType,
+        expected_out: str | None = None,
+    ) -> None:
         """Run some concurrency testing code and see that it was all covered.
 
         `code` is the Python code to execute.  `concurrency` is the name of
@@ -238,39 +245,40 @@ class ConcurrencyTest(CoverageTest):
             # If the test fails, it's helpful to see this info:
             fname = abs_file("try_it.py")
             linenos = data.lines(fname)
+            assert linenos is not None
             print(f"{len(linenos)}: {linenos}")
             print_simple_annotation(code, linenos)
 
             lines = line_count(code)
             assert line_counts(data)['try_it.py'] == lines
 
-    def test_threads(self):
+    def test_threads(self) -> None:
         code = (THREAD + SUM_RANGE_Q + PRINT_SUM_RANGE).format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "thread", threading)
 
-    def test_threads_simple_code(self):
+    def test_threads_simple_code(self) -> None:
         code = SIMPLE.format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "thread", threading)
 
-    def test_eventlet(self):
+    def test_eventlet(self) -> None:
         code = (EVENTLET + SUM_RANGE_Q + PRINT_SUM_RANGE).format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "eventlet", eventlet)
 
-    def test_eventlet_simple_code(self):
+    def test_eventlet_simple_code(self) -> None:
         code = SIMPLE.format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "eventlet", eventlet)
 
     # https://github.com/nedbat/coveragepy/issues/663
     @pytest.mark.skipif(env.WINDOWS, reason="gevent has problems on Windows: #663")
-    def test_gevent(self):
+    def test_gevent(self) -> None:
         code = (GEVENT + SUM_RANGE_Q + PRINT_SUM_RANGE).format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "gevent", gevent)
 
-    def test_gevent_simple_code(self):
+    def test_gevent_simple_code(self) -> None:
         code = SIMPLE.format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "gevent", gevent)
 
-    def test_greenlet(self):
+    def test_greenlet(self) -> None:
         GREENLET = """\
             from greenlet import greenlet
 
@@ -288,11 +296,11 @@ class ConcurrencyTest(CoverageTest):
             """
         self.try_some_code(GREENLET, "greenlet", greenlet, "hello world\n42\n")
 
-    def test_greenlet_simple_code(self):
+    def test_greenlet_simple_code(self) -> None:
         code = SIMPLE.format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "greenlet", greenlet)
 
-    def test_bug_330(self):
+    def test_bug_330(self) -> None:
         BUG_330 = """\
             from weakref import WeakKeyDictionary
             import eventlet
@@ -310,7 +318,8 @@ class ConcurrencyTest(CoverageTest):
             """
         self.try_some_code(BUG_330, "eventlet", eventlet, "0\n")
 
-    def test_threads_with_gevent(self):
+    @flaky_method(max_runs=3)   # Sometimes a test fails due to inherent randomness. Try more times.
+    def test_threads_with_gevent(self) -> None:
         self.make_file("both.py", """\
             import queue
             import threading
@@ -339,7 +348,7 @@ class ConcurrencyTest(CoverageTest):
                 "Couldn't trace with concurrency=gevent, the module isn't installed.\n"
             )
             pytest.skip("Can't run test without gevent installed.")
-        if not env.C_TRACER:
+        if not testenv.C_TRACER:
             assert out == (
                 "Can't support concurrency=gevent with PyTracer, only threads are supported.\n"
             )
@@ -351,25 +360,25 @@ class ConcurrencyTest(CoverageTest):
         last_line = self.squeezed_lines(out)[-1]
         assert re.search(r"TOTAL \d+ 0 100%", last_line)
 
-    def test_bad_concurrency(self):
+    def test_bad_concurrency(self) -> None:
         with pytest.raises(ConfigError, match="Unknown concurrency choices: nothing"):
             self.command_line("run --concurrency=nothing prog.py")
 
-    def test_bad_concurrency_in_config(self):
+    def test_bad_concurrency_in_config(self) -> None:
         self.make_file(".coveragerc", "[run]\nconcurrency = nothing\n")
         with pytest.raises(ConfigError, match="Unknown concurrency choices: nothing"):
             self.command_line("run prog.py")
 
-    def test_no_multiple_light_concurrency(self):
+    def test_no_multiple_light_concurrency(self) -> None:
         with pytest.raises(ConfigError, match="Conflicting concurrency settings: eventlet, gevent"):
             self.command_line("run --concurrency=gevent,eventlet prog.py")
 
-    def test_no_multiple_light_concurrency_in_config(self):
+    def test_no_multiple_light_concurrency_in_config(self) -> None:
         self.make_file(".coveragerc", "[run]\nconcurrency = gevent, eventlet\n")
         with pytest.raises(ConfigError, match="Conflicting concurrency settings: eventlet, gevent"):
             self.command_line("run prog.py")
 
-    def test_multiprocessing_needs_config_file(self):
+    def test_multiprocessing_needs_config_file(self) -> None:
         with pytest.raises(ConfigError, match="multiprocessing requires a configuration file"):
             self.command_line("run --concurrency=multiprocessing prog.py")
 
@@ -378,9 +387,9 @@ class WithoutConcurrencyModuleTest(CoverageTest):
     """Tests of what happens if the requested concurrency isn't installed."""
 
     @pytest.mark.parametrize("module", ["eventlet", "gevent", "greenlet"])
-    def test_missing_module(self, module):
+    def test_missing_module(self, module: str) -> None:
         self.make_file("prog.py", "a = 1")
-        sys.modules[module] = None
+        sys.modules[module] = None      # type: ignore[assignment]
         msg = f"Couldn't trace with concurrency={module}, the module isn't installed."
         with pytest.raises(ConfigError, match=msg):
             self.command_line(f"run --concurrency={module} prog.py")
@@ -427,37 +436,36 @@ MULTI_CODE = """
         for pid, sq in outputs:
             pids.add(pid)
             total += sq
-        print("%d pids, total = %d" % (len(pids), total))
+        print(f"{{len(pids)}} pids, {{total = }}")
         pool.close()
         pool.join()
     """
 
 
 @pytest.fixture(params=["fork", "spawn"], name="start_method")
-def start_method_fixture(request):
+def start_method_fixture(request: pytest.FixtureRequest) -> str:
     """Parameterized fixture to choose the start_method for multiprocessing."""
-    start_method = request.param
+    start_method: str = request.param
     if start_method not in multiprocessing.get_all_start_methods():
         # Windows doesn't support "fork".
         pytest.skip(f"start_method={start_method} not supported here")
     return start_method
 
 
-@pytest.mark.skipif(not multiprocessing, reason="No multiprocessing in this Python")
-@flaky(max_runs=30)         # Sometimes a test fails due to inherent randomness. Try more times.
+#@flaky(max_runs=30)         # Sometimes a test fails due to inherent randomness. Try more times.
 class MultiprocessingTest(CoverageTest):
     """Test support of the multiprocessing module."""
 
     def try_multiprocessing_code(
         self,
-        code,
-        expected_out,
-        the_module,
-        nprocs,
-        start_method,
-        concurrency="multiprocessing",
-        args="",
-    ):
+        code: str,
+        expected_out: str | None,
+        the_module: ModuleType,
+        nprocs: int,
+        start_method: str,
+        concurrency: str = "multiprocessing",
+        args: str = "",
+    ) -> None:
         """Run code using multiprocessing, it should produce `expected_out`."""
         self.make_file("multi.py", code)
         self.make_file(".coveragerc", f"""\
@@ -466,9 +474,7 @@ class MultiprocessingTest(CoverageTest):
             source = .
             """)
 
-        cmd = "coverage run {args} multi.py {start_method}".format(
-            args=args, start_method=start_method,
-        )
+        cmd = f"coverage run {args} multi.py {start_method}"
         out = self.run_command(cmd)
         expected_cant_trace = cant_trace_msg(concurrency, the_module)
 
@@ -484,20 +490,24 @@ class MultiprocessingTest(CoverageTest):
             out_lines = out.splitlines()
             assert len(out_lines) == nprocs + 1
             assert all(
-                re.fullmatch(r"Combined data file \.coverage\..*\.\d+\.\d+", line)
+                re.fullmatch(
+                    r"(Combined data file|Skipping duplicate data) \.coverage\..*\.\d+\.X\w{6}x",
+                    line,
+                )
                 for line in out_lines
             )
+            assert len(glob.glob(".coverage.*")) == 0
             out = self.run_command("coverage report -m")
 
             last_line = self.squeezed_lines(out)[-1]
             assert re.search(r"TOTAL \d+ 0 100%", last_line)
 
-    def test_multiprocessing_simple(self, start_method):
+    def test_multiprocessing_simple(self, start_method: str) -> None:
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
         total = sum(x*x if x%2 else x*x*x for x in range(upto))
-        expected_out = f"{nprocs} pids, total = {total}"
+        expected_out = f"{nprocs} pids, {total = }"
         self.try_multiprocessing_code(
             code,
             expected_out,
@@ -506,7 +516,7 @@ class MultiprocessingTest(CoverageTest):
             start_method=start_method,
         )
 
-    def test_multiprocessing_append(self, start_method):
+    def test_multiprocessing_append(self, start_method: str) -> None:
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
@@ -521,7 +531,7 @@ class MultiprocessingTest(CoverageTest):
             start_method=start_method,
         )
 
-    def test_multiprocessing_and_gevent(self, start_method):
+    def test_multiprocessing_and_gevent(self, start_method: str) -> None:
         nprocs = 3
         upto = 30
         code = (
@@ -538,7 +548,7 @@ class MultiprocessingTest(CoverageTest):
             start_method=start_method,
         )
 
-    def test_multiprocessing_with_branching(self, start_method):
+    def test_multiprocessing_with_branching(self, start_method: str) -> None:
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
@@ -562,7 +572,7 @@ class MultiprocessingTest(CoverageTest):
         last_line = self.squeezed_lines(out)[-1]
         assert re.search(r"TOTAL \d+ 0 \d+ 0 100%", last_line)
 
-    def test_multiprocessing_bootstrap_error_handling(self):
+    def test_multiprocessing_bootstrap_error_handling(self) -> None:
         # An exception during bootstrapping will be reported.
         self.make_file("multi.py", """\
             import multiprocessing
@@ -577,9 +587,9 @@ class MultiprocessingTest(CoverageTest):
             """)
         out = self.run_command("coverage run multi.py")
         assert "Exception during multiprocessing bootstrap init" in out
-        assert "Exception: Crashing because called by _bootstrap" in out
+        assert "RuntimeError: Crashing because called by _bootstrap" in out
 
-    def test_bug_890(self):
+    def test_bug_890(self) -> None:
         # chdir in multiprocessing shouldn't keep us from finding the
         # .coveragerc file.
         self.make_file("multi.py", """\
@@ -599,11 +609,12 @@ class MultiprocessingTest(CoverageTest):
         assert out.splitlines()[-1] == "ok"
 
 
-def test_coverage_stop_in_threads():
+@pytest.mark.skipif(not testenv.SETTRACE_CORE, reason="gettrace is not supported with this core.")
+def test_coverage_stop_in_threads() -> None:
     has_started_coverage = []
     has_stopped_coverage = []
 
-    def run_thread():           # pragma: nested
+    def run_thread() -> None:           # pragma: nested
         """Check that coverage is stopping properly in threads."""
         deadline = time.time() + 5
         ident = threading.current_thread().ident
@@ -617,40 +628,39 @@ def test_coverage_stop_in_threads():
         has_stopped_coverage.append(ident)
 
     cov = coverage.Coverage()
-    cov.start()
+    with cov.collect():
+        t = threading.Thread(target=run_thread)
+        t.start()
 
-    t = threading.Thread(target=run_thread)             # pragma: nested
-    t.start()                                           # pragma: nested
-
-    time.sleep(0.1)                                     # pragma: nested
-    cov.stop()                                          # pragma: nested
+        time.sleep(0.1)
     t.join()
 
     assert has_started_coverage == [t.ident]
     assert has_stopped_coverage == [t.ident]
 
 
-def test_thread_safe_save_data(tmpdir):
+def test_thread_safe_save_data(tmp_path: pathlib.Path) -> None:
     # Non-regression test for: https://github.com/nedbat/coveragepy/issues/581
 
     # Create some Python modules and put them in the path
-    modules_dir = tmpdir.mkdir('test_modules')
+    modules_dir = tmp_path / "test_modules"
+    modules_dir.mkdir()
     module_names = [f"m{i:03d}" for i in range(1000)]
     for module_name in module_names:
-        modules_dir.join(module_name + ".py").write("def f(): pass\n")
+        (modules_dir / (module_name + ".py")).write_text("def f(): pass\n")
 
     # Shared variables for threads
     should_run = [True]
     imported = []
 
     old_dir = os.getcwd()
-    os.chdir(modules_dir.strpath)
+    os.chdir(modules_dir)
     try:
         # Make sure that all dummy modules can be imported.
         for module_name in module_names:
             import_local_file(module_name)
 
-        def random_load():                              # pragma: nested
+        def random_load() -> None:                      # pragma: nested
             """Import modules randomly to stress coverage."""
             while should_run[0]:
                 module_name = random.choice(module_names)
@@ -664,16 +674,13 @@ def test_thread_safe_save_data(tmpdir):
         duration = 0.01
         for _ in range(3):
             cov = coverage.Coverage()
-            cov.start()
+            with cov.collect():
+                threads = [threading.Thread(target=random_load) for _ in range(10)]
+                should_run[0] = True
+                for t in threads:
+                    t.start()
 
-            threads = [threading.Thread(target=random_load) for _ in range(10)]     # pragma: nested
-            should_run[0] = True                    # pragma: nested
-            for t in threads:                       # pragma: nested
-                t.start()
-
-            time.sleep(duration)                    # pragma: nested
-
-            cov.stop()                              # pragma: nested
+                time.sleep(duration)
 
             # The following call used to crash with running background threads.
             cov.get_data()
@@ -697,7 +704,7 @@ class SigtermTest(CoverageTest):
     """Tests of our handling of SIGTERM."""
 
     @pytest.mark.parametrize("sigterm", [False, True])
-    def test_sigterm_saves_data(self, sigterm):
+    def test_sigterm_multiprocessing_saves_data(self, sigterm: bool) -> None:
         # A terminated process should save its coverage data.
         self.make_file("clobbered.py", """\
             import multiprocessing
@@ -725,14 +732,12 @@ class SigtermTest(CoverageTest):
             [run]
             parallel = True
             concurrency = multiprocessing
-            """ + ("sigterm = true" if sigterm else "")
+            """ + ("sigterm = true" if sigterm else ""),
             )
         out = self.run_command("coverage run clobbered.py")
-        # Under the Python tracer on Linux, we get the "Trace function changed"
-        # message. Does that matter?
-        if "Trace function changed" in out:
+        # Under Linux, things go wrong. Does that matter?
+        if env.LINUX and "assert self._collectors" in out:
             lines = out.splitlines(True)
-            assert len(lines) == 5  # "trace function changed" and "self.warn("
             out = "".join(lines[:3])
         assert out == "START\nNOT THREE\nEND\n"
         self.run_command("coverage combine")
@@ -743,7 +748,33 @@ class SigtermTest(CoverageTest):
             expected = "clobbered.py 17 5 71% 5-10"
         assert self.squeezed_lines(out)[2] == expected
 
-    def test_sigterm_still_runs(self):
+    def test_sigterm_threading_saves_data(self) -> None:
+        # A terminated process should save its coverage data.
+        self.make_file("handler.py", """\
+            import os, signal
+
+            print("START", flush=True)
+            print("SIGTERM", flush=True)
+            os.kill(os.getpid(), signal.SIGTERM)
+            print("NOT HERE", flush=True)
+            """)
+        self.make_file(".coveragerc", """\
+            [run]
+            # The default concurrency option.
+            concurrency = thread
+            sigterm = true
+            """)
+        out = self.run_command("coverage run handler.py")
+        out_lines = out.splitlines()
+        assert len(out_lines) in [2, 3]
+        assert out_lines[:2] == ["START", "SIGTERM"]
+        if len(out_lines) == 3:
+            assert out_lines[2] == "Terminated"
+        out = self.run_command("coverage report -m")
+        expected = "handler.py 5 1 80% 6"
+        assert self.squeezed_lines(out)[2] == expected
+
+    def test_sigterm_still_runs(self) -> None:
         # A terminated process still runs its own SIGTERM handler.
         self.make_file("handler.py", """\
             import multiprocessing
